
	page	60,132

        Title   KEYPAD.ASM Matrix keypad driver
        Subttl  Copyright JK microsystems 1997-2002

;************************************************************************;
;*								       	*;
;*      Copyright JK microsystems 1997-2002  -  All Rights Reserved     *;
;*									*;
;*	This software is NOT shareware, freeware, or public domain.	*;
;*	It is the property of JK microsystems.				*;
;*									*;
;*	Customers of JK microsystems may modify the source code		*; 
;*	and/or distribute the binary image of this software without 	*;
;*	additional costs provided it is run only on hardware 		*;
;*	manufactured by JK microsystems.  All other use is expressly 	*;
;*	prohibited.							*;
;*									*;
;*	Update Log							*;
;*									*;
;*	Version	Date		Comments		Progammer	*;
;*	-------	-------		----------------------	----------	*;
;*	1.0	06-10-97	Initial release		jds		*;
;*	2.0	12-17-97	modified KEPAD.ASM			*;
;*				for 386ex		jds		*;
;*	2.1	02-12-99        Fixed problem with bios			*;
;*				32d and later				*;
;*							jds		*;
;*      3.0     06-23-00        modified KEYPADED.ASM                   *;
;*                              for LogicFlex           ew              *;
;*                                                                      *;
;*      4.0     08-20-02        modified KEYPADLF.ASM                   *;
;*                              for Flashlite 186       ew              *;

;************************************************************************;

Code_seg	Segment
	assume	cs:code_seg,ds:code_seg,ss:code_seg

	org	100h
start:					; Put the initialization code at
	jmp	init			; the end so we don't have to save it
	

;-----------------------------------------------------------------------------
;
;	This is invoked with each bios int16h keyboard interrupt
;
key_int:
	cmp	cs:[state],1		; is there a char pending
	jne	key_int_done

	cmp	ah,0			; function 0?
	jne	check_func_1		; no, check 1

	mov	ah,0			; zero the scan code
	mov	al,cs:[buffer]		; get the char in al
	mov	cs:[state],2		; set state to check for key up
	iret				; and return		

check_func_1:
	cmp	ah,1			; function 1?
	jne	key_int_done		; no, give the old handler a shot

	cmp	al,al			; set the zero flag
	mov	ah,0			; zero the scan code
	mov 	al,cs:[buffer]		; get the char, don't change state
	iret


key_int_done:	
	db	0EAh			; far jump, masm too stupid to do it
key_off	dw	0			; to old key isr
key_seg	dw	0		


;-----------------------------------------------------------------------------
;
;	This is invoked with each bios tick interrupt
;
tick_int:
	push	ax			; save the context
	push	bx		
	push	si
	push	di
	push	ds

	cmp	cs:[state],0		; idle
	je	state0
	cmp	cs:[state],1		; char ready, waiting for int16
	je	state1
	cmp	cs:[state],2		; key down, waiting for up
	je	state2		

	mov	cs:[state],0		; unknown state, make it idle	

tick_int_done:
	pop	ds
	pop	di
	pop	si
	pop	bx
	pop	ax

	 db	0EAh			; far jump
tick_off dw	0			; to old tick isr
tick_seg dw	0


;-----------------------------------------------------------------------------
;
;	State 0 - Idle, scan the keypad
;
;	If key pressed, put it in buffer and set state to 1
;	If not pressed, stay in state 0
;
state0:
	call	scan			; scan the keypad
	jc	tick_int_done		; carry set, no keypress
	mov	cs:[buffer],al		; put the char in the buffer
	mov	cs:[state],1		; set state 1
	jmp	tick_int_done		; and leave


;-----------------------------------------------------------------------------
;
;	State 1 - Char in buffer, waiting for int16 to get it
;
state1:
	jmp	tick_int_done		; nothing to do here


;-----------------------------------------------------------------------------
;
;	State 2 - Waiting for key to be unpressed
;
state2:
	call	scan			; scan the keypad
	jnc	tick_int_done		; key is down, stay in state
	mov	cs:[state],0		; key up, set state 0
	jmp	tick_int_done		; done

;-----------------------------------------------------------------------------
;
;	Scan the keypad, return row and column in BL and BH respectively.
;
;	Carry set if key up, carry cleared if key down
;
Scan:
	push	dx
        mov     si,603h                  ; port PD
        mov     di,602h                  ; port PC

	mov	al,0FEh			; scan bit 0 to start
	mov	bh,0			; col counter

column_loop:	
	mov	bl,0			; row counter

	mov	dx,si
	out	dx,al			; drop the current column line

	push	ax			; a small time delay for 
	mov	al,10h			; the comparator to read the new

comparator_delay:			; value
	dec	al
	jnz 	comparator_delay
	pop	ax

	mov	dx,di
	push	bx
	push	ax
	in	al,dx
	mov	bl,al
	pop	ax
	mov	ah,bl
	pop	bx

row_loop:
	shr	ah,1			; shift the tested row into carry
	jnc	scan_done		; key down, all done

	inc	bl			; increment row counter
	cmp	bl,rows			; done?
	jne	row_loop		; no, back through

	inc	bh			; increment column counter
	cmp	bh,columns		; done?
	rol	al,1			; rotate the column line zero bit
	jne	column_loop		; no, back through
	
	stc				; set carry, no key press

scan_done:			
	pushf				; save carry

	shl	bl,1			; put the row/col in one byte
	shl	bl,1
	shl	bl,1
	
	mov	ax,0
	and	bl,038h
	and	bh,07h
	or	al,bl
	or	al,bh

	mov	si,offset keymap	; point to map
	add	si,ax			; add in the offset
	popf				; restore carry
	mov	al,cs:[si]		; get the ascii char
	pop	dx
	ret


;-----------------------------------------------------------------------------
;
;	Data Area
;
state	db	0			
buffer	db	0			; key code 

rows	equ	4			; number of rows to scan
columns equ	4			; number of columns
		
keymap:
	org	keymap+38h
	db			0
	org	keymap+30h
	db			0
	org	keymap+28h
	db			0
	org	keymap+20h
	db			0
	org	keymap+18h
        db                      '1','2','3','A'         ; 64-0001
;        db                      '1','2','3','F'        ; 64-0000
	org	keymap+10h
        db                      '4','5','6','B'         ; 64-0001
;        db                      '4','5','6','E'        ; 64-0000
	org	keymap+8h	
        db                      '7','8','9','C'         ; 64-0001 
;        db                      '7','8','9','D'        ; 64-0000 
	org	keymap		
        db                      '*','0','#','D'         ; 64-0001          
;        db                      'A','0','B','C'        ; 64-0000

	org	keymap+40h

bitmap:
bit0	db	11111111b
bit1	db	11111110b
bit2	db	11111100b
bit3	db	11111000b
bit4	db	11110000b
bit5	db	11100000b
bit6	db	11000000b
bit7	db	10000000b
bit8	db	00000000b


;-----------------------------------------------------------------------------
;
;	This is our initialization code
;
init:
        mov     dx,606h                 ; point to Port Config Reg
        in      al,dx                   ; get current contents
        or      al,08h                  ; set Port D as output
        out     dx,al                   ; do it

	mov	ax,0			; point to interrupt vector table
	mov 	ds,ax

	cli				; no ints while changing vectors

	mov	ax,ds:[16h * 4]		; get int 16h offset
	mov	cs:[key_off],ax		; save it and replace
	mov	ds:[16h * 4],offset key_int 

	mov	ax,ds:[16h * 4 + 2]	; get int 16h segment
	mov	cs:[key_seg],ax		; save it
	mov	ax,cs			; get our segment
	mov	ds:[16h * 4 + 2],ax	; and install it

	mov	ax,ds:[1Ch * 4]		; get int 1Ch offset
	mov	cs:[tick_off],ax	; save it and replace
	mov	ds:[1Ch * 4],offset tick_int 

	mov	ax,ds:[1Ch * 4 + 2]	; get int 1Ch segment
	mov	cs:[tick_seg],ax	; save it
	mov	ax,cs			; get our segment
	mov	ds:[1Ch * 4 + 2],ax	; and install it

	sti				; ints back on

	mov	ax,cs			; put ds back
	mov	ds,ax

	mov	ax,3100h		; TSR function code
	mov 	dx,offset init	 	; get last address used
	mov 	cl,4			; convert to paragraphs
	shr     dx,cl			; by dividing by 16
	inc	dx			; and rounding up by one

	int	21h			; tell dos to do it

code_seg 	ends

	end	start

